;*************************************************************************
;** PROC: pixelate()
;** PURP: To put an image onto the screen from VGA memory w/ pitching
;** USAG: pixelate(char *buffer,int x,int y,int ratio);
;**       Ratio is a word.  The value is in a ratio of 256.  So,
;**       the ratio of 520 would result in 49% of the original size.
;**       (In other words, 256/Ratio = New Size)
;**
;**       Buffer is broken down as follows:
;**       X Size (2), Y Size (2), then the image. 
;**
;**       NOTE:  No clipping is done for this, and it is currently
;**              only written to support chained mode.  I am rewriting
;**              it to perform clipping and support for unchained video
;**              modes.  Questions or comments email:
;**            
;**              ketrenoj@ucs.orst.edu
;**              -=[ Lord Logics ]=-      
;**       Group: None.  Want me?  Contact me . . .
;**
;*************************************************************************
pixelate        proc far
        push    bp
        mov     bp,sp
        push    es
        push    ds
        push    si
        push    di
 
        cld
 
        mov     ax,@data
        mov     ds,ax
 
        mov     di,[bp+8]       ; Set ES:[DI] to point to the buffer . . .
        mov     es,di           ; :
        mov     di,[bp+6]       ; :
 
        mov     dx,es:[di]      ; X_SIZE is stored in DX
        mov     cx,es:[di+2]    ; Y_SIZE is stored in CX
 
        mov     ax,[bp+14]      ; We will now do some adjustemnts via ratio
        push    dx              ; Set up the ratio factor to adjust the
        push    ax              ; : X_SIZE and Y_SIZE, and X and Y POS.
        mul     dx              ; :
        mov     bx,0100h        ; :
        div     bx              ; :
        mov     X_SIZE,ax       ; : Newly adjusted X_SIZE (after pitching)
        pop     ax              ; :
        mul     cx              ; :
        div     bx              ; :
        mov     Y_SIZE,ax       ; : Newly adjusted Y_SIZE (after pitching)
        pop     dx              ; :
 
;** Now check to make sure that there is indeed something to display . . .
        cmp     X_SIZE,0        ; Check to see if there is still an image.
        jz      px_d            ; :
        cmp     Y_SIZE,0        ; :
        jz      px_d            ; :

;** Now we will figure out how much the X and Y POS will be changed . . .
;** On entry:
;** AX - Nothing
;** BX - Nothing
;** CX - Y_SIZE (no pitching)
;** DX - X_SIZE (no pitching)
;** ES:[DI] - Image Buffer
;** DS:[SI] - Nothing
px_sx:  sub     dx,X_SIZE       ; Set up the X_SIZE variance . . .
        jge     px_xn           ; : Variance = 1/2 the size difference . . .
        neg     dx              ; :
        shr     dx,1            ; :
        neg     dx              ; :
        jmp     px_sy           ; :
px_xn:  shr     dx,1            ; :
px_sy:  sub     cx,Y_SIZE       ; Set up the Y_SIZE variance . . .
        jge     px_yn           ; :
        neg     cx              ; :
        shr     cx,1            ; :
        neg     cx              ; :
        jmp     px_si           ; :
px_yn:  shr     cx,1            ; :
 
px_si:  xor     bx,bx           ; Clear initial DI displacement.
        push    dx
        mov     ax,[bp+12]      ; Set up the starting point and check
        add     ax,cx           ; : to make sure that the image will fit
        cmp     ax,200          ; : within our bounds (ie, the screen).
        jge     px_d            ; :
        cmp     ax,0            ; :
        jge     px_s1           ; :
        push    ax              ; :
        neg     ax              ; :
        sub     Y_SIZE,ax       ; :             THE FOLLOWING MAY CAUSE
        jle     px_ddx          ; :             PROBLEMS DURING CLIPPING.
        mov     dx,es:[di]      ; : Set DX to X_SIZE (no pitching)
        push    ax              ; :
        mul     dx              ; :
        mov     bx,ax           ; : BX stores DI adjustment . . .
        mov     dx,320          ; :
        pop     ax              ; :
        mul     dx              ; :
        mov     si,ax           ; :
        pop     ax              ; :
        jmp     px_s2           ; :
 
;**
;** Exit the procedure . . .
;**
px_ddx: pop     dx
px_dax: pop     ax
px_d:   pop     di
        pop     si
        pop     ds
        pop     es
        pop     bp
        ret
 
px_s1:  mov     dx,320          ; Adjust SI for proper positioning.
        mul     dx              ; :
        mov     si,ax           ; :
 
px_s2:  pop     dx              ; Now do the X Adjusting . . .
        mov     ax,[bp+10]      ; Set up the starting point and check
        add     ax,dx           ; : to make sure that the image will fit
        cmp     ax,320          ; : within our bounds (ie, the screen).
        jge     px_d            ; :
        cmp     ax,0            ; :
        jge     px_s3           ; :
        neg     ax              ; :
        sub     X_SIZE,ax       ; :
        jle     px_d            ; :
        add     bx,ax           ; :
        jmp     px_s4           ; :
 
px_s3:  add     si,ax           ; Set up screen position adjustment
px_s4:  mov     cx,bx
        mov     bx,es:[di]
        add     di,cx
        add     di,4
        mov     cx,Y_SIZE
        mov     dx,X_SIZE
        jmp     px_pixel

;**
;** Display the image as quick as possible.
;**
;** On entry:
;** ES:[DI] Points to image buffer
;** DS:[SI] Points to screen memory location (start point)
;** AX - nothing important
;** BX - X_SIZE w/out any pitching.  This is used to add to the memory
;**      buffer for each line displayed.
;** CX - Y_COUNT size.  The number of Y_LINES to display
;** DX - X_COUNT size.  The number of X_LINES to display
;**
px_pixel:
        ;** First we must set up our ratio skipping . . .
        push    dx              ; Save X_COUNT
        push    bx              ; Save X_SIZE
        mov     bx,[bp+14]      ; Set skipping bytes . . .
        xor     dx,dx           ; :
        mov     ax,0FFFFh       ; :
        or      bx,bx           ; :
        jz      px_pd           ; :
        div     bx              ; :
        jmp     px_pf           ; :
px_pd:  mov     ax,0100h        ; :
px_pf:  mov     [bp+14],ax      ; :
        pop     bx              ; Restore X_SIZE
        pop     dx              ; Restore X_COUNT
 
 
        ;** Now we must set up our starting points . . .
        mov     ax,es           ; Set DS:[SI] to our image,
        mov     ds,ax           ; : ES:[DI] to the screen position,
        mov     ax,0A000h       ; : BP to point to the ratio skip,
        mov     es,ax           ; : and AX to the skip, roll over . . .
        xchg    si,di           ; :
        add     bp,14           ; :
        mov     al,[bp+1]       ; :
        xor     ah,ah           ; :
        mov     [bp-2],ah       ; :
 
        ;** Now, actually display the image via to for/next loops . . .
pxq_y:  push    cx              ; Save the Y_COUNT
        push    ax              ; Save the Whole Adjustment
        push    bx              ; Save the X_SIZE (no pitching)
        push    dx              ; Save the X_COUNT
        push    si              ; Save the start of line position
 
        ;** Now we will loop through the X_LINE, first setting up the
        ;** : values needed internally though . . .
        mov     cx,dx           ; Set CX to X_COUNT
        mov     dl,[bp]         ; Set DL to Ratio
        xor     dh,dh           ; Clear the Spill Over
 
pxq_x:  movsb                   ; Move one byte
        dec     si              ; : Fix the SI from being incremented.
        add     dh,dl           ; Do pitching . . .
        adc     si,ax           ; :
        loop    pxq_x           ; Do the entire X_LINE
 
        ;** Now we will adjust for the next X_LINE along the Y_LINE . . .
        pop     si              ; Restore start of line position.
        pop     bx              ; Adjust the dest screen position . . .
        add     di,320          ; :
        sub     di,bx           ; :
 
        add     [bp-2],dl       ; Do the pitching . . .
        adc     ax,0            ; :
        mov     dx,ax           ; :
        pop     ax              ; : Restore AX to the X_SIZE
        mov     cx,bx           ; : Set CX to X_COUNT
        mov     bx,ax           ; : Set BX to X_SIZE
        mul     dx              ; :
        add     si,ax           ; :
        pop     ax              ; : Restore Whole Adjustment
        mov     dx,cx           ; : Set DX to X_COUNT
        pop     cx              ; Restore Y_COUNT
        loop    pxq_y           ; Do the entire Y_LINE.
        jmp     px_d            ; Exit the PIXELATE routine.
pixelate        endp            ; END OF PIXELATE